@@ -5,17 +5,18 @@
 
 //! An implementation of the Triple Diffie-Hellman key exchange protocol
 use crate::{
-    ciphersuite::CipherSuite,
     errors::{
         utils::{check_slice_size, check_slice_size_atleast},
         InternalPakeError, PakeError, ProtocolError,
     },
     group::Group,
     hash::Hash,
-    key_exchange::traits::{FromBytes, KeyExchange, ToBytes, ToBytesWithPointers},
-    keypair::{KeyPair, PrivateKey, PublicKey, SizedBytesExt},
-    serialization::serialize,
+    key_exchange::traits::{KeyExchange, ToBytes, ToBytesWithPointers},
+    keypair::{Key, KeyPair, SizedBytesExt},
+    serialization::{serialize, tokenize},
 };
+use alloc::vec::Vec;
+use core::convert::TryFrom;
 use digest::{Digest, FixedOutput};
 use generic_array::{
     typenum::{Unsigned, U32},
@@ -25,18 +26,19 @@ use generic_bytes::SizedBytes;
 use hkdf::Hkdf;
 use hmac::{Hmac, Mac, NewMac};
 use rand::{CryptoRng, RngCore};
-use std::convert::TryFrom;
 use zeroize::Zeroize;
 
 const KEY_LEN: usize = 32;
 pub(crate) type NonceLen = U32;
 
-static STR_RFC: &[u8] = b"RFCXXXX";
-static STR_CLIENT_MAC: &[u8] = b"ClientMAC";
-static STR_HANDSHAKE_SECRET: &[u8] = b"HandshakeSecret";
-static STR_SERVER_MAC: &[u8] = b"ServerMAC";
-static STR_SESSION_KEY: &[u8] = b"SessionKey";
-static STR_OPAQUE: &[u8] = b"OPAQUE-";
+static STR_3DH: &[u8] = b"3DH";
+static STR_CLIENT_MAC: &[u8] = b"client mac";
+static STR_HANDSHAKE_SECRET: &[u8] = b"handshake secret";
+static STR_SERVER_MAC: &[u8] = b"server mac";
+static STR_HANDSHAKE_ENC: &[u8] = b"handshake enc";
+static STR_ENCRYPTION_PAD: &[u8] = b"encryption pad";
+static STR_SESSION_SECRET: &[u8] = b"session secret";
+static STR_OPAQUE: &[u8] = b"OPAQUE ";
 
 #[allow(clippy::upper_case_acronyms)]
 /// The Triple Diffie-Hellman key exchange implementation
@@ -50,6 +52,7 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
     type KE3Message = Ke3Message<<D as FixedOutput>::OutputSize>;
 
     fn generate_ke1<R: RngCore + CryptoRng>(
+        info: Vec<u8>,
         rng: &mut R,
     ) -> Result<(Self::KE1State, Self::KE1Message), ProtocolError> {
         let client_e_kp = KeyPair::<G>::generate_random(rng);
@@ -57,6 +60,7 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
 
         let ke1_message = Ke1Message {
             client_nonce,
+            info,
             client_e_pk: client_e_kp.public().clone(),
         };
 
@@ -75,26 +79,25 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
         serialized_credential_request: Vec<u8>,
         l2_bytes: Vec<u8>,
         ke1_message: Self::KE1Message,
-        client_s_pk: PublicKey,
-        server_s_sk: PrivateKey,
+        client_s_pk: Key,
+        server_s_sk: Key,
         id_u: Vec<u8>,
         id_s: Vec<u8>,
-        context: Vec<u8>,
-    ) -> Result<(Self::KE2State, Self::KE2Message), ProtocolError> {
+        e_info: Vec<u8>,
+    ) -> Result<(Vec<u8>, Self::KE2State, Self::KE2Message), ProtocolError> {
         let server_e_kp = KeyPair::<G>::generate_random(rng);
         let server_nonce = generate_nonce::<R>(rng);
 
         let mut transcript_hasher = D::new()
-            .chain(STR_RFC)
-            .chain(&serialize(&context, 2))
-            .chain(&id_u)
+            .chain(STR_3DH)
+            .chain(&serialize(&id_u, 2)?)
             .chain(&serialized_credential_request[..])
-            .chain(&id_s)
+            .chain(&serialize(&id_s, 2)?)
             .chain(&l2_bytes[..])
             .chain(&server_nonce[..])
             .chain(&server_e_kp.public().to_arr());
 
-        let (session_key, km2, km3) = derive_3dh_keys::<D, G>(
+        let (session_key, km2, ke2, km3) = derive_3dh_keys::<D, G>(
             TripleDHComponents {
                 pk1: ke1_message.client_e_pk.clone(),
                 sk1: server_e_kp.private().clone(),
@@ -106,6 +109,19 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
             &transcript_hasher.clone().finalize(),
         )?;
 
+        // Compute encryption of e_info
+        let h = Hkdf::<D>::from_prk(&ke2).map_err(|_| InternalPakeError::HkdfError)?;
+        let mut encryption_pad = alloc::vec![0u8; e_info.len()];
+        h.expand(STR_ENCRYPTION_PAD, &mut encryption_pad)
+            .map_err(|_| InternalPakeError::HkdfError)?;
+        let ciphertext: Vec<u8> = encryption_pad
+            .iter()
+            .zip(e_info.iter())
+            .map(|(&x1, &x2)| x1 ^ x2)
+            .collect();
+
+        transcript_hasher.update(&serialize(&ciphertext, 2)?);
+
         let mut mac_hasher =
             Hmac::<D>::new_from_slice(&km2).map_err(|_| InternalPakeError::HmacError)?;
         mac_hasher.update(&transcript_hasher.clone().finalize());
@@ -114,6 +130,7 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
         transcript_hasher.update(&mac);
 
         Ok((
+            ke1_message.info,
             Ke2State {
                 km3,
                 hashed_transcript: transcript_hasher.finalize(),
@@ -122,6 +139,7 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
             Ke2Message {
                 server_nonce,
                 server_e_pk: server_e_kp.public().clone(),
+                e_info: ciphertext,
                 mac,
             },
         ))
@@ -133,22 +151,20 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
         ke2_message: Self::KE2Message,
         ke1_state: &Self::KE1State,
         serialized_credential_request: &[u8],
-        server_s_pk: PublicKey,
-        client_s_sk: PrivateKey,
+        server_s_pk: Key,
+        client_s_sk: Key,
         id_u: Vec<u8>,
         id_s: Vec<u8>,
-        context: Vec<u8>,
-    ) -> Result<(Vec<u8>, Self::KE3Message), ProtocolError> {
+    ) -> Result<(Vec<u8>, Vec<u8>, Self::KE3Message), ProtocolError> {
         let mut transcript_hasher = D::new()
-            .chain(STR_RFC)
-            .chain(&serialize(&context, 2))
-            .chain(&id_u)
+            .chain(STR_3DH)
+            .chain(&serialize(&id_u, 2)?)
             .chain(&serialized_credential_request)
-            .chain(&id_s)
+            .chain(&serialize(&id_s, 2)?)
             .chain(&l2_component[..])
             .chain(&ke2_message.to_bytes_without_info_or_mac());
 
-        let (session_key, km2, km3) = derive_3dh_keys::<D, G>(
+        let (session_key, km2, ke2, km3) = derive_3dh_keys::<D, G>(
             TripleDHComponents {
                 pk1: ke2_message.server_e_pk.clone(),
                 sk1: ke1_state.client_e_sk.clone(),
@@ -160,6 +176,8 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
             &transcript_hasher.clone().finalize(),
         )?;
 
+        transcript_hasher.update(&serialize(&ke2_message.e_info[..], 2)?);
+
         let mut server_mac =
             Hmac::<D>::new_from_slice(&km2).map_err(|_| InternalPakeError::HmacError)?;
         server_mac.update(&transcript_hasher.clone().finalize());
@@ -170,13 +188,25 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
             ));
         }
 
-        transcript_hasher.update(ke2_message.mac);
+        transcript_hasher.update(ke2_message.mac.to_vec());
 
         let mut client_mac =
             Hmac::<D>::new_from_slice(&km3).map_err(|_| InternalPakeError::HmacError)?;
         client_mac.update(&transcript_hasher.finalize());
 
+        // Compute decryption of e_info
+        let h = Hkdf::<D>::from_prk(&ke2).map_err(|_| InternalPakeError::HkdfError)?;
+        let mut encryption_pad = alloc::vec![0u8; ke2_message.e_info.len()];
+        h.expand(STR_ENCRYPTION_PAD, &mut encryption_pad)
+            .map_err(|_| InternalPakeError::HkdfError)?;
+        let plaintext: Vec<u8> = encryption_pad
+            .iter()
+            .zip(ke2_message.e_info.iter())
+            .map(|(&x1, &x2)| x1 ^ x2)
+            .collect();
+
         Ok((
+            plaintext,
             session_key.to_vec(),
             Ke3Message {
                 mac: client_mac.finalize().into_bytes(),
@@ -208,29 +238,30 @@ impl<D: Hash, G: Group> KeyExchange<D, G> for TripleDH {
 }
 
 /// The client state produced after the first key exchange message
-#[derive(PartialEq, Eq, Debug, Hash, Zeroize, Clone)]
-#[cfg_attr(feature = "serialize", derive(serde::Deserialize, serde::Serialize))]
+#[derive(PartialEq, Eq, Zeroize)]
 #[zeroize(drop)]
 pub struct Ke1State {
-    client_e_sk: PrivateKey,
+    client_e_sk: Key,
     client_nonce: GenericArray<u8, NonceLen>,
 }
 
 /// The first key exchange message
-#[derive(PartialEq, Eq, Debug, Hash, Clone)]
-#[cfg_attr(feature = "serialize", derive(serde::Deserialize, serde::Serialize))]
+#[derive(PartialEq, Eq)]
 pub struct Ke1Message {
     pub(crate) client_nonce: GenericArray<u8, NonceLen>,
-    pub(crate) client_e_pk: PublicKey,
+    pub(crate) info: Vec<u8>,
+    pub(crate) client_e_pk: Key,
 }
 
-impl FromBytes for Ke1State {
-    fn from_bytes<CS: CipherSuite>(bytes: &[u8]) -> Result<Self, PakeError> {
+impl TryFrom<&[u8]> for Ke1State {
+    type Error = PakeError;
+
+    fn try_from(bytes: &[u8]) -> Result<Self, Self::Error> {
         let nonce_len = NonceLen::to_usize();
         let checked_bytes = check_slice_size_atleast(bytes, KEY_LEN + nonce_len, "ke1_state")?;
 
         Ok(Self {
-            client_e_sk: PrivateKey::from_bytes(&checked_bytes[..KEY_LEN])?,
+            client_e_sk: Key::from_bytes(&checked_bytes[..KEY_LEN])?,
             client_nonce: GenericArray::clone_from_slice(
                 &checked_bytes[KEY_LEN..KEY_LEN + nonce_len],
             ),
@@ -239,17 +270,17 @@ impl FromBytes for Ke1State {
 }
 
 impl ToBytesWithPointers for Ke1State {
-    fn to_bytes(&self) -> Vec<u8> {
+    fn to_bytes(&self) -> Result<Vec<u8>, ProtocolError> {
         let output: Vec<u8> = [&self.client_e_sk.to_arr(), &self.client_nonce[..]].concat();
-        output
+        Ok(output)
     }
 
     #[cfg(test)]
     fn as_byte_ptrs(&self) -> Vec<(*const u8, usize)> {
-        vec![
+        alloc::vec![
             (
                 self.client_e_sk.as_ptr(),
-                <PrivateKey as SizedBytes>::Len::to_usize(),
+                <Key as SizedBytes>::Len::to_usize(),
             ),
             (self.client_nonce.as_ptr(), NonceLen::to_usize()),
         ]
@@ -257,27 +288,36 @@ impl ToBytesWithPointers for Ke1State {
 }
 
 impl ToBytes for Ke1Message {
-    fn to_bytes(&self) -> Vec<u8> {
-        [&self.client_nonce[..], &self.client_e_pk.to_arr()].concat()
+    fn to_bytes(&self) -> Result<Vec<u8>, ProtocolError> {
+        Ok([
+            &self.client_nonce[..],
+            &serialize(&self.info, 2)?,
+            &self.client_e_pk.to_arr(),
+        ]
+        .concat())
     }
 }
 
-impl FromBytes for Ke1Message {
-    fn from_bytes<CS: CipherSuite>(ke1_message_bytes: &[u8]) -> Result<Self, PakeError> {
+impl TryFrom<&[u8]> for Ke1Message {
+    type Error = PakeError;
+
+    fn try_from(ke1_message_bytes: &[u8]) -> Result<Self, Self::Error> {
         let nonce_len = NonceLen::to_usize();
         let checked_nonce =
-            check_slice_size(ke1_message_bytes, nonce_len + KEY_LEN, "ke1_message nonce")?;
+            check_slice_size_atleast(ke1_message_bytes, nonce_len, "ke1_message nonce")?;
+
+        let (info, remainder) = tokenize(&checked_nonce[nonce_len..], 2)?;
+
+        let checked_client_e_pk = check_slice_size(&remainder, KEY_LEN, "ke1_message client_e_pk")?;
 
         Ok(Self {
             client_nonce: GenericArray::clone_from_slice(&checked_nonce[..nonce_len]),
-            client_e_pk: PublicKey::from_bytes(&checked_nonce[nonce_len..])?,
+            info,
+            client_e_pk: Key::from_bytes(checked_client_e_pk)?,
         })
     }
 }
 /// The server state produced after the second key exchange message
-#[derive(Clone, Debug, Eq, Hash, PartialEq)]
-#[cfg_attr(feature = "serialize", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "serialize", serde(bound = ""))]
 pub struct Ke2State<HashLen: ArrayLength<u8>> {
     km3: GenericArray<u8, HashLen>,
     hashed_transcript: GenericArray<u8, HashLen>,
@@ -300,18 +340,18 @@ impl<HashLen: ArrayLength<u8>> Drop for Ke2State<HashLen> {
 }
 
 impl<HashLen: ArrayLength<u8>> ToBytesWithPointers for Ke2State<HashLen> {
-    fn to_bytes(&self) -> Vec<u8> {
-        [
+    fn to_bytes(&self) -> Result<Vec<u8>, ProtocolError> {
+        Ok([
             &self.km3[..],
             &self.hashed_transcript[..],
             &self.session_key[..],
         ]
-        .concat()
+        .concat())
     }
 
     #[cfg(test)]
     fn as_byte_ptrs(&self) -> Vec<(*const u8, usize)> {
-        vec![
+        alloc::vec![
             (self.km3.as_ptr(), HashLen::to_usize()),
             (self.hashed_transcript.as_ptr(), HashLen::to_usize()),
             (self.session_key.as_ptr(), HashLen::to_usize()),
@@ -320,17 +360,18 @@ impl<HashLen: ArrayLength<u8>> ToBytesWithPointers for Ke2State<HashLen> {
 }
 
 /// The second key exchange message
-#[derive(Clone, Debug, Eq, Hash, PartialEq)]
-#[cfg_attr(feature = "serialize", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "serialize", serde(bound = ""))]
+#[derive(Clone)]
 pub struct Ke2Message<HashLen: ArrayLength<u8>> {
     server_nonce: GenericArray<u8, NonceLen>,
-    server_e_pk: PublicKey,
+    server_e_pk: Key,
+    e_info: Vec<u8>,
     mac: GenericArray<u8, HashLen>,
 }
 
-impl<HashLen: ArrayLength<u8>> FromBytes for Ke2State<HashLen> {
-    fn from_bytes<CS: CipherSuite>(input: &[u8]) -> Result<Self, PakeError> {
+impl<HashLen: ArrayLength<u8>> TryFrom<&[u8]> for Ke2State<HashLen> {
+    type Error = PakeError;
+
+    fn try_from(input: &[u8]) -> Result<Self, Self::Error> {
         let hash_len = HashLen::to_usize();
         let checked_bytes = check_slice_size(input, 3 * hash_len, "ke2_state")?;
 
@@ -345,8 +386,13 @@ impl<HashLen: ArrayLength<u8>> FromBytes for Ke2State<HashLen> {
 }
 
 impl<HashLen: ArrayLength<u8>> ToBytes for Ke2Message<HashLen> {
-    fn to_bytes(&self) -> Vec<u8> {
-        [&self.to_bytes_without_info_or_mac(), &self.mac[..]].concat()
+    fn to_bytes(&self) -> Result<Vec<u8>, ProtocolError> {
+        Ok([
+            &self.to_bytes_without_info_or_mac(),
+            &serialize(&self.e_info, 2)?,
+            &self.mac[..],
+        ]
+        .concat())
     }
 }
 
@@ -356,30 +402,24 @@ impl<HashLen: ArrayLength<u8>> Ke2Message<HashLen> {
     }
 }
 
-impl<HashLen: ArrayLength<u8>> FromBytes for Ke2Message<HashLen> {
-    fn from_bytes<CS: CipherSuite>(input: &[u8]) -> Result<Self, PakeError> {
+impl<HashLen: ArrayLength<u8>> TryFrom<&[u8]> for Ke2Message<HashLen> {
+    type Error = PakeError;
+
+    fn try_from(input: &[u8]) -> Result<Self, Self::Error> {
         let nonce_len = NonceLen::to_usize();
         let checked_nonce = check_slice_size_atleast(input, nonce_len, "ke2_message nonce")?;
-
-        let unchecked_server_e_pk = check_slice_size_atleast(
+        let checked_server_e_pk = check_slice_size_atleast(
             &checked_nonce[nonce_len..],
             KEY_LEN,
             "ke2_message server_e_pk",
         )?;
-        let checked_mac = check_slice_size(
-            &unchecked_server_e_pk[KEY_LEN..],
-            HashLen::to_usize(),
-            "ke1_message mac",
-        )?;
-
-        // Check the public key bytes
-        let server_e_pk = KeyPair::<CS::Group>::check_public_key(PublicKey::from_bytes(
-            &unchecked_server_e_pk[..KEY_LEN],
-        )?)?;
+        let (e_info, remainder) = tokenize(&checked_server_e_pk[KEY_LEN..], 2)?;
+        let checked_mac = check_slice_size(&remainder, HashLen::to_usize(), "ke1_message mac")?;
 
         Ok(Self {
             server_nonce: GenericArray::clone_from_slice(&checked_nonce[..nonce_len]),
-            server_e_pk: PublicKey::from_bytes(&server_e_pk)?,
+            server_e_pk: Key::from_bytes(&checked_server_e_pk[..KEY_LEN])?,
+            e_info,
             mac: GenericArray::clone_from_slice(checked_mac),
         })
     }
@@ -388,38 +428,38 @@ impl<HashLen: ArrayLength<u8>> FromBytes for Ke2Message<HashLen> {
 #[allow(clippy::upper_case_acronyms)]
 // The triple of public and private components used in the 3DH computation
 struct TripleDHComponents {
-    pk1: PublicKey,
-    sk1: PrivateKey,
-    pk2: PublicKey,
-    sk2: PrivateKey,
-    pk3: PublicKey,
-    sk3: PrivateKey,
+    pk1: Key,
+    sk1: Key,
+    pk2: Key,
+    sk2: Key,
+    pk3: Key,
+    sk3: Key,
 }
 
 #[allow(clippy::upper_case_acronyms)]
-// Consists of a session key, followed by two mac keys: (session_key, km2, km3)
+// Consists of a session key, followed by two mac keys and an encryption key: (session_key, km2, ke2, km3)
 type TripleDHDerivationResult<D> = (
     GenericArray<u8, <D as FixedOutput>::OutputSize>,
     GenericArray<u8, <D as FixedOutput>::OutputSize>,
     GenericArray<u8, <D as FixedOutput>::OutputSize>,
+    GenericArray<u8, <D as FixedOutput>::OutputSize>,
 );
 
 /// The third key exchange message
-#[derive(Clone, Debug, Eq, Hash, PartialEq)]
-#[cfg_attr(feature = "serialize", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "serialize", serde(bound = ""))]
 pub struct Ke3Message<HashLen: ArrayLength<u8>> {
     mac: GenericArray<u8, HashLen>,
 }
 
 impl<HashLen: ArrayLength<u8>> ToBytes for Ke3Message<HashLen> {
-    fn to_bytes(&self) -> Vec<u8> {
-        self.mac.to_vec()
+    fn to_bytes(&self) -> Result<Vec<u8>, ProtocolError> {
+        Ok(self.mac.to_vec())
     }
 }
 
-impl<HashLen: ArrayLength<u8>> FromBytes for Ke3Message<HashLen> {
-    fn from_bytes<CS: CipherSuite>(bytes: &[u8]) -> Result<Self, PakeError> {
+impl<HashLen: ArrayLength<u8>> TryFrom<&[u8]> for Ke3Message<HashLen> {
+    type Error = PakeError;
+
+    fn try_from(bytes: &[u8]) -> Result<Self, Self::Error> {
         let checked_bytes = check_slice_size(bytes, HashLen::to_usize(), "ke3_message")?;
 
         Ok(Self {
@@ -451,7 +491,7 @@ fn derive_3dh_keys<D: Hash, G: Group>(
     )?;
     let session_key = derive_secrets::<D>(
         &extracted_ikm,
-        STR_SESSION_KEY,
+        STR_SESSION_SECRET,
         hashed_derivation_transcript,
     )?;
 
@@ -461,6 +501,12 @@ fn derive_3dh_keys<D: Hash, G: Group>(
         b"",
         <D as Digest>::OutputSize::to_usize(),
     )?;
+    let ke2 = hkdf_expand_label::<D>(
+        &handshake_secret,
+        STR_HANDSHAKE_ENC,
+        b"",
+        <D as Digest>::OutputSize::to_usize(),
+    )?;
     let km3 = hkdf_expand_label::<D>(
         &handshake_secret,
         STR_CLIENT_MAC,
@@ -471,6 +517,7 @@ fn derive_3dh_keys<D: Hash, G: Group>(
     Ok((
         GenericArray::clone_from_slice(&session_key),
         GenericArray::clone_from_slice(&km2),
+        GenericArray::clone_from_slice(&ke2),
         GenericArray::clone_from_slice(&km3),
     ))
 }
@@ -491,7 +538,7 @@ fn hkdf_expand_label_extracted<D: Hash>(
     context: &[u8],
     length: usize,
 ) -> Result<Vec<u8>, ProtocolError> {
-    let mut okm = vec![0u8; length];
+    let mut okm = alloc::vec![0u8; length];
 
     let mut hkdf_label: Vec<u8> = Vec::new();
 
@@ -501,9 +548,9 @@ fn hkdf_expand_label_extracted<D: Hash>(
     let mut opaque_label: Vec<u8> = Vec::new();
     opaque_label.extend_from_slice(STR_OPAQUE);
     opaque_label.extend_from_slice(label);
-    hkdf_label.extend_from_slice(&serialize(&opaque_label, 1));
+    hkdf_label.extend_from_slice(&serialize(&opaque_label, 1)?);
 
-    hkdf_label.extend_from_slice(&serialize(context, 1));
+    hkdf_label.extend_from_slice(&serialize(context, 1)?);
 
     hkdf.expand(&hkdf_label, &mut okm)
         .map_err(|_| InternalPakeError::HkdfError)?;
@@ -525,7 +572,7 @@ fn derive_secrets<D: Hash>(
 
 // Generate a random nonce up to NonceLen::to_usize() bytes.
 fn generate_nonce<R: RngCore + CryptoRng>(rng: &mut R) -> GenericArray<u8, NonceLen> {
-    let mut nonce_bytes = vec![0u8; NonceLen::to_usize()];
+    let mut nonce_bytes = alloc::vec![0u8; NonceLen::to_usize()];
     rng.fill_bytes(&mut nonce_bytes);
     GenericArray::clone_from_slice(&nonce_bytes)
 }
