@@ -9,18 +9,19 @@
 
 use crate::errors::InternalPakeError;
 use crate::group::Group;
+use alloc::vec::Vec;
+use core::fmt::Debug;
+use core::marker::PhantomData;
+use core::ops::Deref;
 #[cfg(test)]
 use generic_array::typenum::Unsigned;
 use generic_array::{typenum::U32, GenericArray};
 use generic_bytes::{SizedBytes, TryFromSizedBytesError};
-#[cfg(test)]
+#[cfg(all(test, feature = "std"))]
 use proptest::prelude::*;
-#[cfg(test)]
+#[cfg(all(test, feature = "std"))]
 use rand::{rngs::StdRng, SeedableRng};
 use rand::{CryptoRng, RngCore};
-use std::fmt::Debug;
-use std::marker::PhantomData;
-use std::ops::Deref;
 use zeroize::Zeroize;
 
 /// Convenience extension trait of SizedBytes
@@ -71,7 +72,7 @@ impl<G: Group> KeyPair<G> {
     pub(crate) fn generate_random<R: RngCore + CryptoRng>(rng: &mut R) -> Self {
         let sk = G::random_nonzero_scalar(rng);
         let sk_bytes = G::scalar_as_bytes(&sk);
-        let pk = G::base_point().mult_by_slice(&sk_bytes);
+        let pk = G::base_point().mult_by_slice(sk_bytes);
         Self {
             pk: Key(pk.to_arr().to_vec()),
             sk: Key(sk_bytes.to_vec()),
@@ -83,7 +84,7 @@ impl<G: Group> KeyPair<G> {
     /// &public_from_private(self.private()) == self.public()
     pub(crate) fn public_from_private(bytes: &Key) -> Key {
         let bytes_data = GenericArray::<u8, G::ScalarLen>::from_slice(&bytes.0[..]);
-        Key(G::base_point().mult_by_slice(&bytes_data).to_arr().to_vec())
+        Key(G::base_point().mult_by_slice(bytes_data).to_arr().to_vec())
     }
 
     /// Check whether a public key is valid. This is meant to be applied on
@@ -97,14 +98,14 @@ impl<G: Group> KeyPair<G> {
     /// Computes the diffie hellman function on a public key and private key
     pub(crate) fn diffie_hellman(pk: Key, sk: Key) -> Result<Vec<u8>, InternalPakeError> {
         let pk_data = GenericArray::<u8, G::ElemLen>::from_slice(&pk.0[..]);
-        let point = G::from_element_slice(&pk_data)?;
+        let point = G::from_element_slice(pk_data)?;
         let secret_data = GenericArray::<u8, G::ScalarLen>::from_slice(&sk.0[..]);
-        Ok(G::mult_by_slice(&point, &secret_data).to_arr().to_vec())
+        Ok(G::mult_by_slice(&point, secret_data).to_arr().to_vec())
     }
 
     /// Obtains a KeyPair from a slice representing the private key
     pub fn from_private_key_slice(input: &[u8]) -> Result<Self, InternalPakeError> {
-        let sk = Key::from_arr(GenericArray::from_slice(&input))?;
+        let sk = Key::from_arr(GenericArray::from_slice(input))?;
         let pk = Self::public_from_private(&sk);
         Ok(Self {
             pk,
@@ -115,14 +116,14 @@ impl<G: Group> KeyPair<G> {
 
     #[cfg(test)]
     pub fn as_byte_ptrs(&self) -> Vec<(*const u8, usize)> {
-        vec![
+        alloc::vec![
             (self.pk.as_ptr(), <Key as SizedBytes>::Len::to_usize()),
             (self.sk.as_ptr(), <Key as SizedBytes>::Len::to_usize()),
         ]
     }
 }
 
-#[cfg(test)]
+#[cfg(all(test, feature = "std"))]
 impl<G: Group + Debug> KeyPair<G> {
     /// Test-only strategy returning a proptest Strategy based on
     /// generate_random
@@ -170,15 +171,15 @@ impl SizedBytes for Key {
 mod tests {
     use super::*;
     use crate::errors::*;
+    use core::slice::from_raw_parts;
     use curve25519_dalek::ristretto::RistrettoPoint;
     use generic_array::typenum::Unsigned;
     use rand::rngs::OsRng;
-    use std::slice::from_raw_parts;
 
     #[test]
     fn test_zeroize_key() -> Result<(), ProtocolError> {
         let key_len = <Key as SizedBytes>::Len::to_usize();
-        let mut key = Key(vec![1u8; key_len]);
+        let mut key = Key(alloc::vec![1u8; key_len]);
         let ptr = key.as_ptr();
 
         key.zeroize();
@@ -205,6 +206,7 @@ mod tests {
         Ok(())
     }
 
+    #[cfg(feature = "std")]
     proptest! {
         #[test]
         fn test_ristretto_check(kp in KeyPair::<RistrettoPoint>::uniform_keypair_strategy()) {
