@@ -702,6 +702,7 @@
 
 #![cfg_attr(not(feature = "bench"), deny(missing_docs))]
 #![deny(unsafe_code)]
+#![cfg_attr(not(feature = "std"), no_std)]
 
 #[cfg(not(any(feature = "u64_backend", feature = "u32_backend",)))]
 compile_error!(
@@ -709,6 +710,8 @@ compile_error!(
      please enable one of: u64_backend, u32_backend"
 );
 
+extern crate alloc;
+
 // Error types
 pub mod errors;
 
