@@ -9,20 +9,26 @@ use crate::{
     errors::*,
     group::Group,
     key_exchange::{
-        traits::{FromBytes, KeyExchange, ToBytes},
+        traits::{KeyExchange, ToBytes},
         tripledh::{NonceLen, TripleDH},
     },
-    keypair::{KeyPair, PublicKey},
+    opaque::*,
     serialization::{i2osp, os2ip, serialize},
     *,
 };
 
+#[cfg(test)]
+use alloc::vec;
+#[cfg(test)]
+use alloc::vec::Vec;
+
 use curve25519_dalek::{ristretto::RistrettoPoint, traits::Identity};
 use generic_array::typenum::Unsigned;
 use generic_bytes::SizedBytes;
 use proptest::{collection::vec, prelude::*};
 use rand::{rngs::OsRng, RngCore};
 
+use core::convert::TryFrom;
 use sha2::Digest;
 
 struct Default;
@@ -33,7 +39,7 @@ impl CipherSuite for Default {
     type SlowHash = crate::slow_hash::NoOpHash;
 }
 
-const HASH_SIZE: usize = 64; // Because of SHA512
+const MAX_INFO_LENGTH: usize = 10;
 const MAC_SIZE: usize = 64; // Because of SHA512
 
 fn random_ristretto_point() -> RistrettoPoint {
@@ -55,9 +61,10 @@ fn client_registration_roundtrip() {
     let pw = b"hunter2";
     let mut rng = OsRng;
     let sc = <RistrettoPoint as Group>::random_nonzero_scalar(&mut rng);
+    let elem = <RistrettoPoint as Group>::base_point() * sc;
 
-    // serialization order: scalar, password
-    let bytes: Vec<u8> = [&sc.as_bytes()[..], &pw[..]].concat();
+    // serialization order: scalar, password, group element
+    let bytes: Vec<u8> = [&elem.to_arr(), &sc.as_bytes()[..], &pw[..]].concat();
     let reg = ClientRegistration::<Default>::deserialize(&bytes[..]).unwrap();
     let reg_bytes = reg.serialize();
     assert_eq!(reg_bytes, bytes);
@@ -68,20 +75,28 @@ fn server_registration_roundtrip() {
     // If we don't have envelope and client_pk, the server registration just
     // contains the prf key
     let mut rng = OsRng;
-    let mut masking_key = [0u8; HASH_SIZE];
-    rng.fill_bytes(&mut masking_key);
+    let oprf_key = <RistrettoPoint as Group>::random_nonzero_scalar(&mut rng);
+    let mut oprf_bytes: Vec<u8> = vec![];
+    oprf_bytes.extend_from_slice(oprf_key.as_bytes());
+    let reg = ServerRegistration::<Default>::deserialize(&oprf_bytes[..]).unwrap();
+    let reg_bytes = reg.serialize();
+    assert_eq!(reg_bytes, oprf_bytes);
+
+    let mut ciphertext = [0u8; 32];
+    rng.fill_bytes(&mut ciphertext);
 
     // Construct a mock envelope
     let mut mock_envelope_bytes = Vec::new();
+    mock_envelope_bytes.extend_from_slice(&[1; 1]); // mode = 1
     mock_envelope_bytes.extend_from_slice(&vec![0; NonceLen::to_usize()]); // empty nonce
-                                                                           // mock_envelope_bytes.extend_from_slice(&ciphertext); // ciphertext which is an encrypted private key
+    mock_envelope_bytes.extend_from_slice(&ciphertext); // ciphertext which is an encrypted private key
     mock_envelope_bytes.extend_from_slice(&[0; MAC_SIZE]); // length-MAC_SIZE hmac
 
-    let mock_client_kp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let mock_client_kp = Default::generate_random_keypair(&mut rng);
     // serialization order: oprf_key, public key, envelope
     let mut bytes = Vec::<u8>::new();
+    bytes.extend_from_slice(oprf_key.as_bytes());
     bytes.extend_from_slice(&mock_client_kp.public().to_arr());
-    bytes.extend_from_slice(&masking_key);
     bytes.extend_from_slice(&mock_envelope_bytes);
     let reg = ServerRegistration::<Default>::deserialize(&bytes[..]).unwrap();
     let reg_bytes = reg.serialize();
@@ -117,7 +132,7 @@ fn registration_response_roundtrip() {
     let pt = random_ristretto_point();
     let beta_bytes = pt.to_arr();
     let mut rng = OsRng;
-    let skp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let skp = Default::generate_random_keypair(&mut rng);
     let pubkey_bytes = skp.public().to_arr();
 
     let mut input = Vec::new();
@@ -143,25 +158,27 @@ fn registration_response_roundtrip() {
 #[test]
 fn registration_upload_roundtrip() {
     let mut rng = OsRng;
-    let skp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let skp = Default::generate_random_keypair(&mut rng);
     let pubkey_bytes = skp.public().to_arr();
 
     let mut key = [0u8; 32];
     rng.fill_bytes(&mut key);
-    let mut nonce = [0u8; 32];
-    rng.fill_bytes(&mut nonce);
 
-    let mut masking_key = vec![0u8; <sha2::Sha512 as Digest>::OutputSize::to_usize()];
-    rng.fill_bytes(&mut masking_key);
+    let mut msg = [0u8; 32];
+    rng.fill_bytes(&mut msg);
 
-    let (envelope, _) =
-        Envelope::<Default>::seal_raw(&key, &nonce, &pubkey_bytes, InnerEnvelopeMode::Internal)
-            .unwrap();
+    let (envelope, _) = Envelope::<sha2::Sha512>::seal_raw(
+        &mut rng,
+        &key,
+        &msg,
+        &pubkey_bytes,
+        InnerEnvelopeMode::Base,
+    )
+    .unwrap();
     let envelope_bytes = envelope.serialize();
 
     let mut input = Vec::new();
     input.extend_from_slice(&pubkey_bytes[..]);
-    input.extend_from_slice(&masking_key[..]);
     input.extend_from_slice(&envelope_bytes);
 
     let r3 = RegistrationUpload::<Default>::deserialize(&input[..]).unwrap();
@@ -175,18 +192,26 @@ fn credential_request_roundtrip() {
     let alpha = random_ristretto_point();
     let alpha_bytes = alpha.to_arr().to_vec();
 
-    let client_e_kp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let client_e_kp = Default::generate_random_keypair(&mut rng);
     let mut client_nonce = vec![0u8; NonceLen::to_usize()];
     rng.fill_bytes(&mut client_nonce);
 
-    let ke1m: Vec<u8> = [&client_nonce[..], &client_e_kp.public()].concat();
+    let mut info = [0u8; MAX_INFO_LENGTH];
+    rng.fill_bytes(&mut info);
+
+    let ke1m: Vec<u8> = [
+        &client_nonce[..],
+        &serialize(&info.to_vec(), 2).unwrap(),
+        &client_e_kp.public(),
+    ]
+    .concat();
 
     let mut input = Vec::new();
     input.extend_from_slice(&alpha_bytes);
     input.extend_from_slice(&ke1m[..]);
 
     let l1 = CredentialRequest::<Default>::deserialize(input.as_slice()).unwrap();
-    let l1_bytes = l1.serialize();
+    let l1_bytes = l1.serialize().unwrap();
     assert_eq!(input, l1_bytes);
 
     // Assert that identity group element is rejected
@@ -207,30 +232,51 @@ fn credential_response_roundtrip() {
     let pt_bytes = pt.to_arr().to_vec();
 
     let mut rng = OsRng;
+    let skp = Default::generate_random_keypair(&mut rng);
+    let pubkey_bytes = skp.public().to_arr();
 
-    let mut masking_nonce = vec![0u8; 32];
-    rng.fill_bytes(&mut masking_nonce);
+    let mut key = [0u8; 32];
+    rng.fill_bytes(&mut key);
 
-    let mut masked_response =
-        vec![0u8; <PublicKey as SizedBytes>::Len::to_usize() + Envelope::<Default>::len()];
-    rng.fill_bytes(&mut masked_response);
+    let mut msg = [0u8; 32];
+    rng.fill_bytes(&mut msg);
 
-    let server_e_kp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let (envelope, _) = Envelope::<sha2::Sha512>::seal_raw(
+        &mut rng,
+        &key,
+        &msg,
+        &pubkey_bytes,
+        InnerEnvelopeMode::Base,
+    )
+    .unwrap();
+
+    let server_e_kp = Default::generate_random_keypair(&mut rng);
     let mut mac = [0u8; MAC_SIZE];
     rng.fill_bytes(&mut mac);
     let mut server_nonce = vec![0u8; NonceLen::to_usize()];
     rng.fill_bytes(&mut server_nonce);
 
-    let ke2m: Vec<u8> = [&server_nonce[..], &server_e_kp.public(), &mac[..]].concat();
+    let mut e_info = [0u8; MAX_INFO_LENGTH];
+    rng.fill_bytes(&mut e_info);
+
+    let ke2m: Vec<u8> = [
+        &server_nonce[..],
+        &server_e_kp.public(),
+        &serialize(&e_info.to_vec(), 2).unwrap(),
+        &mac[..],
+    ]
+    .concat();
+
+    let serialized_envelope = envelope.serialize();
 
     let mut input = Vec::new();
     input.extend_from_slice(pt_bytes.as_slice());
-    input.extend_from_slice(&masking_nonce);
-    input.extend_from_slice(&masked_response);
+    input.extend_from_slice(&pubkey_bytes.as_slice());
+    input.extend_from_slice(&serialized_envelope);
     input.extend_from_slice(&ke2m[..]);
 
     let l2 = CredentialResponse::<Default>::deserialize(&input).unwrap();
-    let l2_bytes = l2.serialize();
+    let l2_bytes = l2.serialize().unwrap();
     assert_eq!(input, l2_bytes);
 
     // Assert that identity group element is rejected
@@ -240,8 +286,8 @@ fn credential_response_roundtrip() {
     assert!(match CredentialResponse::<Default>::deserialize(
         &[
             identity_bytes,
-            masking_nonce.to_vec(),
-            masked_response,
+            pubkey_bytes.to_vec(),
+            serialized_envelope,
             ke2m.to_vec()
         ]
         .concat()
@@ -260,7 +306,7 @@ fn login_third_message_roundtrip() {
     let input: Vec<u8> = [&mac[..]].concat();
 
     let l3 = CredentialFinalization::<Default>::deserialize(&input).unwrap();
-    let l3_bytes = l3.serialize();
+    let l3_bytes = l3.serialize().unwrap();
     assert_eq!(input, l3_bytes);
 }
 
@@ -270,7 +316,7 @@ fn client_login_roundtrip() {
     let mut rng = OsRng;
     let sc = <RistrettoPoint as Group>::random_nonzero_scalar(&mut rng);
 
-    let client_e_kp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let client_e_kp = Default::generate_random_keypair(&mut rng);
     let mut client_nonce = vec![0u8; NonceLen::to_usize()];
     rng.fill_bytes(&mut client_nonce);
 
@@ -280,13 +326,13 @@ fn client_login_roundtrip() {
     // serialization order: scalar, credential_request, ke1_state, password
     let bytes: Vec<u8> = [
         &sc.as_bytes()[..],
-        &serialize(&serialized_credential_request, 2),
-        &serialize(&l1_data, 2),
+        &serialize(&serialized_credential_request, 2).unwrap(),
+        &serialize(&l1_data, 2).unwrap(),
         &pw[..],
     ]
     .concat();
     let reg = ClientLogin::<Default>::deserialize(&bytes[..]).unwrap();
-    let reg_bytes = reg.serialize();
+    let reg_bytes = reg.serialize().unwrap();
     assert_eq!(reg_bytes, bytes);
 }
 
@@ -294,16 +340,23 @@ fn client_login_roundtrip() {
 fn ke1_message_roundtrip() {
     let mut rng = OsRng;
 
-    let client_e_kp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let client_e_kp = Default::generate_random_keypair(&mut rng);
     let mut client_nonce = vec![0u8; NonceLen::to_usize()];
     rng.fill_bytes(&mut client_nonce);
 
-    let ke1m: Vec<u8> = [&client_nonce[..], &client_e_kp.public()].concat();
-    let reg = <TripleDH as KeyExchange<sha2::Sha512, RistrettoPoint>>::KE1Message::from_bytes::<
-        Default,
-    >(&ke1m[..])
-    .unwrap();
-    let reg_bytes = reg.to_bytes();
+    let mut info = [0u8; MAX_INFO_LENGTH];
+    rng.fill_bytes(&mut info);
+
+    let ke1m: Vec<u8> = [
+        &client_nonce[..],
+        &serialize(&info.to_vec(), 2).unwrap(),
+        &client_e_kp.public(),
+    ]
+    .concat();
+    let reg =
+        <TripleDH as KeyExchange<sha2::Sha512, RistrettoPoint>>::KE1Message::try_from(&ke1m[..])
+            .unwrap();
+    let reg_bytes = reg.to_bytes().unwrap();
     assert_eq!(reg_bytes, ke1m);
 }
 
@@ -311,19 +364,26 @@ fn ke1_message_roundtrip() {
 fn ke2_message_roundtrip() {
     let mut rng = OsRng;
 
-    let server_e_kp = KeyPair::<<Default as CipherSuite>::Group>::generate_random(&mut rng);
+    let server_e_kp = Default::generate_random_keypair(&mut rng);
     let mut mac = [0u8; MAC_SIZE];
     rng.fill_bytes(&mut mac);
     let mut server_nonce = vec![0u8; NonceLen::to_usize()];
     rng.fill_bytes(&mut server_nonce);
+    let mut e_info = [0u8; MAX_INFO_LENGTH];
+    rng.fill_bytes(&mut e_info);
 
-    let ke2m: Vec<u8> = [&server_nonce[..], &server_e_kp.public(), &mac[..]].concat();
+    let ke2m: Vec<u8> = [
+        &server_nonce[..],
+        &server_e_kp.public(),
+        &serialize(&e_info.to_vec(), 2).unwrap(),
+        &mac[..],
+    ]
+    .concat();
 
-    let reg = <TripleDH as KeyExchange<sha2::Sha512, RistrettoPoint>>::KE2Message::from_bytes::<
-        Default,
-    >(&ke2m[..])
-    .unwrap();
-    let reg_bytes = reg.to_bytes();
+    let reg =
+        <TripleDH as KeyExchange<sha2::Sha512, RistrettoPoint>>::KE2Message::try_from(&ke2m[..])
+            .unwrap();
+    let reg_bytes = reg.to_bytes().unwrap();
     assert_eq!(reg_bytes, ke2m);
 }
 
@@ -335,68 +395,67 @@ fn ke3_message_roundtrip() {
 
     let ke3m: Vec<u8> = [&mac[..]].concat();
 
-    let reg = <TripleDH as KeyExchange<sha2::Sha512, RistrettoPoint>>::KE3Message::from_bytes::<
-        Default,
-    >(&ke3m[..])
-    .unwrap();
-    let reg_bytes = reg.to_bytes();
+    let reg =
+        <TripleDH as KeyExchange<sha2::Sha512, RistrettoPoint>>::KE3Message::try_from(&ke3m[..])
+            .unwrap();
+    let reg_bytes = reg.to_bytes().unwrap();
     assert_eq!(reg_bytes, ke3m);
 }
 
 proptest! {
 
 #[test]
-fn test_i2osp_os2ip(ref bytes in vec(prop::num::u8::ANY, 0..std::mem::size_of::<usize>())) {
-    assert_eq!(&i2osp(os2ip(&bytes)?, bytes.len()), bytes);
+fn test_i2osp_os2ip(bytes in vec(any::<u8>(), 0..core::mem::size_of::<usize>())) {
+    assert_eq!(i2osp(os2ip(&bytes).unwrap(), bytes.len()).unwrap(), bytes);
 }
 
 #[test]
-fn test_nocrash_registration_request(ref bytes in vec(prop::num::u8::ANY, 0..200)) {
+fn test_nocrash_registration_request(bytes in vec(any::<u8>(), 0..200)) {
     RegistrationRequest::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_registration_response(ref bytes in vec(prop::num::u8::ANY, 0..200)) {
+fn test_nocrash_registration_response(bytes in vec(any::<u8>(), 0..200)) {
     RegistrationResponse::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_registration_upload(ref bytes in vec(prop::num::u8::ANY, 0..200)) {
+fn test_nocrash_registration_upload(bytes in vec(any::<u8>(), 0..200)) {
     RegistrationUpload::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_credential_request(ref bytes in vec(prop::num::u8::ANY, 0..500)) {
+fn test_nocrash_credential_request(bytes in vec(any::<u8>(), 0..500)) {
     CredentialRequest::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_credential_response(ref bytes in vec(prop::num::u8::ANY, 0..500)) {
+fn test_nocrash_credential_response(bytes in vec(any::<u8>(), 0..500)) {
     CredentialResponse::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_credential_finalization(ref bytes in vec(prop::num::u8::ANY, 0..500)) {
+fn test_nocrash_credential_finalization(bytes in vec(any::<u8>(), 0..500)) {
     CredentialFinalization::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_client_registration(ref bytes in vec(prop::num::u8::ANY, 0..700)) {
+fn test_nocrash_client_registration(bytes in vec(any::<u8>(), 0..700)) {
     ClientRegistration::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_server_registration(ref bytes in vec(prop::num::u8::ANY, 0..700)) {
+fn test_nocrash_server_registration(bytes in vec(any::<u8>(), 0..700)) {
     ServerRegistration::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_client_login(ref bytes in vec(prop::num::u8::ANY, 0..700)) {
+fn test_nocrash_client_login(bytes in vec(any::<u8>(), 0..700)) {
     ClientLogin::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
 #[test]
-fn test_nocrash_server_login(ref bytes in vec(prop::num::u8::ANY, 0..700)) {
+fn test_nocrash_server_login(bytes in vec(any::<u8>(), 0..700)) {
     ServerLogin::<Default>::deserialize(&bytes[..]).map_or(true, |_| true);
 }
 
